# Generated by Django 4.2.7 on 2023-11-26 13:54

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0020_reflectivequestion"),
    ]

    operations = [
        migrations.CreateModel(
            name="SpeechToTextModelOptions",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("model_name", models.CharField(default="base", max_length=200)),
                (
                    "model_type",
                    models.CharField(
                        choices=[("openai", "Openai"), ("offline", "Offline")], default="offline", max_length=200
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.AlterField(
            model_name="chatmodeloptions",
            name="chat_model",
            field=models.CharField(default="mistral-7b-instruct-v0.1.Q4_0.gguf", max_length=200),
        ),
        migrations.AlterField(
            model_name="chatmodeloptions",
            name="model_type",
            field=models.CharField(
                choices=[("openai", "Openai"), ("offline", "Offline")], default="offline", max_length=200
            ),
        ),
    ]
